@@ -4,7 +4,8 @@ use strict;
 use File::Spec::Functions qw(catfile);
 use Array::Diff;
 
-our $VERSION = '0.91';
+our $VERSION = '0.94';
+$VERSION = eval $VERSION; ## no critic
 
 sub order { 100 }
 
@@ -16,10 +17,6 @@ sub analyse {
     my $class=shift;
     my $me=shift;
     
-    my @files=@{$me->d->{files_array} || []};
-    if (my $ignore = $me->d->{ignored_files_array}) {
-        push @files, @$ignore;
-    }
     my $distdir=$me->distdir;
     my $manifest_file=catfile($distdir,'MANIFEST');
 
@@ -41,7 +38,7 @@ sub analyse {
         close $fh;
 
         @manifest=sort @manifest;
-        my @files=sort @files;
+        my @files=sort keys %{$me->d->{files_hash} || {}};
 
         my $diff=Array::Diff->diff(\@manifest,\@files);
         if ($diff->count == 0) {
@@ -55,6 +52,17 @@ sub analyse {
                 "Missing in Dist: " . join(', ',@{$diff->deleted}));
             $me->d->{error}{manifest_matches_dist} = \@error;
         }
+
+        # Tweak symlinks error for a local distribution (RT #97858)
+        if ($me->d->{is_local_distribution} && $me->d->{error}{symlinks}) {
+            my %manifested = map {$_ => 1} @manifest;
+            my @symlinks = grep {$manifested{$_}} split ',', $me->d->{error}{symlinks};
+            if (@symlinks) {
+                $me->d->{error}{symlinks} = join ',', @symlinks;
+            } else {
+                delete $me->d->{error}{symlinks};
+            }
+        }
     }
     else {
         $me->d->{manifest_matches_dist}=0;
